﻿''' <summary>
''' Mit dieser Klasse kann man sehr leicht eine globale Hotkey funktionalität in seinem Programm einbinden.
''' Man muss nur diese Klasse mit WithEvents deklarieren und ihr eine Form zuweisen die gesubclassed werden soll.
''' Dann muss man nur noch ein paar eigene HotKey-Kombinationen registrieren (z.B. Strg+Alt+X) und diese
''' mit dem Event abfragen bzw, abfangen. Dazu muss man eine eigene HotKeyID angeben um einen bestimmte HotKey
''' Kombination später zu identifizieren wenn diese gedrückt wird. Wenn man z.B. eine Kombination registriert
''' und ihr z.B. die HotKeyID "TEST1" zugewiesen wird, dann kann man später im Event nach dieser ID "TEST1" fragen
''' und dann eine Funktion aufrufen die für diesen HotKey bestimmt wurde.
''' </summary>
''' 
''' <remarks>
''' Ursprung: Tim Hartwig
''' Bearbeitung: Thomas Kriechbaumer
''' </remarks>
Public Class clsHotkeys
    Inherits System.Windows.Forms.NativeWindow
    Implements IDisposable

    Private Declare Function RegisterHotKey Lib "user32" ( _
        ByVal Hwnd As IntPtr, _
        ByVal ID As Integer, _
        ByVal Modifiers As Integer, _
        ByVal Key As Integer) _
    As Integer

    Private Declare Function UnregisterHotKey Lib "user32" ( _
        ByVal Hwnd As IntPtr, _
        ByVal ID As Integer) _
    As Integer

    Public Class HotKeyObject
        Private mHotKey As Keys
        Private mModifier As MODKEY
        Private mHotKeyID As String
      
        Public Property HotKey() As Keys
            Get
                Return mHotKey
            End Get
            Set(ByVal value As Keys)
                mHotKey = value
            End Set
        End Property

        ''' <summary>
        ''' Modifier-Keys, also Strg, Alt, Win und Shift
        ''' </summary>
        ''' <remarks>Können mit OR verknüpft werden</remarks>
        Public Property Modifier() As MODKEY
            Get
                Return mModifier
            End Get
            Set(ByVal value As MODKEY)
                mModifier = value
            End Set
        End Property

        Public Property HotKeyID() As String
            Get
                Return mHotKeyID
            End Get
            Set(ByVal value As String)
                mHotKeyID = value
            End Set
        End Property

        Sub New(ByVal NewHotKey As Keys, ByVal NewModifier As MODKEY, ByVal NewHotKeyID As String)
            mHotKey = NewHotKey
            mModifier = NewModifier
            mHotKeyID = NewHotKeyID
        End Sub
    End Class

    Private Const WM_HOTKEY As Integer = &H312
    Private mHotKeyList As New System.Collections.Generic.List(Of HotKeyObject)

    ''' <summary>
    ''' Diesem Event wird immer die zugewiesene HotKeyID übergeben wenn eine HotKey Kombination gedrückt wurde.
    ''' </summary>
    Public Event HotKeyPressed(ByVal HotKeyID As String)

    Public Enum MODKEY As Integer
        MOD_ALT = 1
        MOD_CONTROL = 2
        MOD_SHIFT = 4
        MOD_WIN = 8
    End Enum

    Sub New()
        ' Wir machen uns unser eigenes Handle.
        Me.CreateHandle(New CreateParams)
    End Sub

    Protected Overrides Sub Finalize()
        ' Das selbst erstellte Handle zerstören.
        Me.DestroyHandle()
        MyBase.Finalize()
    End Sub

    ''' <summary>
    ''' Diese Funktion fügt einen Hotkey hinzu und registriert ihn auch sofort (wenn möglich)
    ''' </summary>
    ''' <param name="new_KeyCode">Den KeyCode für die Taste</param>
    ''' <param name="new_Modifiers">Die Zusatztasten wie z.B. Strg oder Alt, diese können auch mit OR kombiniert werden</param>
    ''' <returns>true wenn der Hotkey erfolgreich registriert wurde, gab es einen Fehler wird false zurückgegeben</returns>
    ''' <param name="new_HotKeyID">Die ID die der Hotkey bekommen soll um diesen zu identifizieren</param>
    Public Function TryAddHotKey(ByVal new_KeyCode As Keys, ByVal new_Modifiers As MODKEY, ByVal new_HotKeyID As String) As Boolean
        For Each hk As HotKeyObject In mHotKeyList
            'es könnte ja sein, dass die Tastenkombination schon registriert ist, aber unter einem anderen Namen
            'oder der Name wurde schon registriert...
            If hk.HotKey = new_KeyCode And hk.Modifier = new_Modifiers Then Return False
            If hk.HotKeyID = new_HotKeyID Then Return False
        Next

        Dim new_ID As Integer = mHotKeyList.Count
        Dim res As Integer
        res = RegisterHotKey(Me.Handle, new_ID, new_Modifiers, new_KeyCode)

        If res <> 0 Then
            mHotKeyList.Add(New HotKeyObject(new_KeyCode, new_Modifiers, new_HotKeyID))
            Return True
        Else
            Return False
        End If
    End Function

    ''' <summary>
    ''' Diese Funktion entfernt einen Hotkey und deregistriert ihn auch sofort
    ''' </summary>
    ''' <param name="HotKeyID">Gibt die HotkeyID an welche entfernt werden soll</param>
    Public Sub RemoveHotKey(ByVal HotKeyID As String)
        Dim index_to_delete As Integer

        If mHotKeyList.Count = 0 Then Exit Sub

        For Each tmp_HotKey As HotKeyObject In mHotKeyList
            If tmp_HotKey.HotKeyID = HotKeyID Then
                index_to_delete = mHotKeyList.IndexOf(tmp_HotKey)
                Exit For
            End If
        Next

        Dim res As Integer
        UnregisterHotKey(Me.Handle, index_to_delete)

        If res <> 0 Then
            mHotKeyList.RemoveAt(index_to_delete)
        Else
            'Der Hotkey konnte nicht deregistriert werden, er verbleibt weiterhin im System (falls er nicht bereits gelöscht wurde)!
        End If
    End Sub

    Protected Overrides Sub WndProc(ByRef m As Message)
        If m.Msg = WM_HOTKEY Then
            RaiseEvent HotKeyPressed(mHotKeyList(CShort(m.WParam)).HotKeyID)
        End If

        MyBase.WndProc(m)
    End Sub

    Private Sub Dispose() Implements IDisposable.Dispose
        Dim res As Integer
        For index_to_delete As Integer = 0 To mHotKeyList.Count - 1
            res = UnregisterHotKey(Me.Handle, index_to_delete)

            'If res = 0 Then
            'Der Hotkey konnte nicht deregistriert werden, er verbleibt weiterhin im System (falls noch vorhanden)!
            'End If
        Next
    End Sub
End Class